;; examples/fileaccess.scm -- file access module for the simple webserver
;;
;; Copyright (C) 2000 Martin Grabmueller <mgrabmue@cs.tu-berlin.de>
;;
;; This is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;; 
;; This software is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;; 
;; You should have received a copy of the GNU General Public License
;; along with this package; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.  


;; Return a true value if access to `filename' should be denied
;; because it contains `..' stuff.  Return #f if access should be
;; granted.
(define (access-forbidden? filename)
  (or (<= (string-length filename) (string-length document-root))
      (not (string=?
	    (substring filename 0 (string-length document-root))
	    document-root))))

(define (file-valid? filename)
  (and (file-exists? filename) (file-regular? filename)))

;; Given an absolute URL, construct a valid local path name for the
;; requested resource.  Append a standard file name if it is a
;; directory.  Return a valid file name or #f if the resource cannot
;; be found.
(define (create-system-filename url)
  (let* ((filename (cleanup-filename (string-append document-root url))))
    (cond 
     ((file-valid? filename) filename)
     ((file-valid? (string-append filename "/index.html"))
      (string-append filename "/index.html"))
     (else #f))))
